                                                                              /******************************************************************************
                               SSD1306OLED.c                                  *
                                                                              *
           SSD1306 OLED driver for mikroC PRO for PIC compiler.               *
   Reference: Adafruit Industries SSD1306 OLED driver and graphics library.   *
                                                                              *
                      The driver is for I2C mode only.                        *
                                                                              *
*******************************************************************************
                                                                              *
 https://simple-circuit.com/                                                  *
                                                                              *
******************************************************************************/



#include <stdbool.h>
#include <stdint.h>
#define Byte unsigned char
#define Word unsigned int           // int16_t


//------------------------------ Definitions ---------------------------------//

#define SSD1306_I2C_ADDRESS   0x78
//#define SSD1306_128_32
#define SSD1306_128_64


#ifdef SSD1306_SOFT_I2C
#define SSD1306_Start    Soft_I2C_Start
#define SSD1306_Write    Soft_I2C_Write
#define SSD1306_Stop     Soft_I2C_Stop
#else
#if !defined SSD1306_I2C1  &&  !defined SSD1306_I2C2
#define  SSD1306_I2C1
#endif

#if defined SSD1306_I2C1  &&  defined SSD1306_I2C2
#undef  SSD1306_I2C2
#endif

#ifdef  SSD1306_I2C1
#define SSD1306_Start    I2C1_Start
#define SSD1306_Write    I2C1_Wr
#define SSD1306_Stop     I2C1_Stop
#endif

#ifdef  SSD1306_I2C2
#define SSD1306_Start    I2C2_Start
#define SSD1306_Write    I2C2_Wr
#define SSD1306_Stop     I2C2_Stop
#endif
#endif



//#if !defined SSD1306_128_32 && !defined SSD1306_96_16
//#define SSD1306_128_64
//#endif
#if defined SSD1306_128_32 && defined SSD1306_96_16
  #error "Only one SSD1306 display can be specified at once"
#endif

#if defined SSD1306_128_64
  #define SSD1306_LCDWIDTH            128
  #define SSD1306_LCDHEIGHT            64
#endif
#if defined SSD1306_128_32
  #define SSD1306_LCDWIDTH            128
  #define SSD1306_LCDHEIGHT            32
#endif
#if defined SSD1306_96_16
  #define SSD1306_LCDWIDTH             96
  #define SSD1306_LCDHEIGHT            16
#endif

#define SSD1306_SETCONTRAST          0x81
#define SSD1306_DISPLAYALLON_RESUME  0xA4
#define SSD1306_DISPLAYALLON         0xA5
#define SSD1306_NORMALDISPLAY        0xA6
#define SSD1306_INVERTDISPLAY_       0xA7
#define SSD1306_DISPLAYOFF           0xAE
#define SSD1306_DISPLAYON            0xAF
#define SSD1306_SETDISPLAYOFFSET     0xD3
#define SSD1306_SETCOMPINS           0xDA
#define SSD1306_SETVCOMDETECT        0xDB
#define SSD1306_SETDISPLAYCLOCKDIV   0xD5
#define SSD1306_SETPRECHARGE         0xD9
#define SSD1306_SETMULTIPLEX         0xA8
#define SSD1306_SETLOWCOLUMN         0x00
#define SSD1306_SETHIGHCOLUMN        0x10
#define SSD1306_SETSTARTLINE         0x40
#define SSD1306_MEMORYMODE           0x20
#define SSD1306_COLUMNADDR           0x21
#define SSD1306_PAGEADDR             0x22
#define SSD1306_COMSCANINC           0xC0
#define SSD1306_COMSCANDEC           0xC8
#define SSD1306_SEGREMAP             0xA0
#define SSD1306_CHARGEPUMP           0x8D
#define SSD1306_EXTERNALVCC          0x01
#define SSD1306_SWITCHCAPVCC         0x02

// Scrolling #defines
#define SSD1306_ACTIVATE_SCROLL                      0x2F
#define SSD1306_DEACTIVATE_SCROLL                    0x2E
#define SSD1306_SET_VERTICAL_SCROLL_AREA             0xA3
#define SSD1306_RIGHT_HORIZONTAL_SCROLL              0x26
#define SSD1306_LEFT_HORIZONTAL_SCROLL               0x27
#define SSD1306_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL 0x29
#define SSD1306_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL  0x2A

extern Byte _i2caddr, _vccstate, x_pos = 0, y_pos = 0, text_size = 1;
extern bool wrap = true, SSD1306_Color = true;

void ssd1306_command(Byte c);
void SSD1306_Begin(Byte vccstate, Byte i2caddr);
void SSD1306_TextSize(Byte t_size);
void SSD1306_GotoXY(Byte x, Byte y);
void SSD1306_DrawPixel(Byte x, Byte y);
void SSD1306_StartScrollRight(Byte start, Byte stop);
void SSD1306_StartScrollLeft(Byte start, Byte stop);
void SSD1306_StartScrollDiagRight(Byte start, Byte stop);
void SSD1306_StartScrollDiagLeft(Byte start, Byte stop);
void SSD1306_StopScroll(void);
void SSD1306_Dim(bool dim);
void SSD1306_Display(void);
void SSD1306_ClearDisplay(void);
void SSD1306_DrawLine(Word x0, Word y0, Word x1, Word y1);
void SSD1306_DrawFastHLine(Byte x, Byte y, Byte w);
void SSD1306_DrawFastVLine(Byte x, Byte y, Byte h);
void SSD1306_FillRect(Byte x, Byte y, Byte w, Byte h);
void SSD1306_FillScreen();
void SSD1306_DrawCircle(Word x0, Word y0, Word r);
void SSD1306_DrawCircleHelper(Word x0, Word y0, Word r, Byte cornername);
void SSD1306_FillCircle(Word x0, Word y0, Word r);
void SSD1306_FillCircleHelper(Word x0, Word y0, Word r, Byte cornername, Word delta);
void SSD1306_DrawRect(Byte x, Byte y, Byte w, Byte h);
void SSD1306_DrawRoundRect(Byte x, Byte y, Byte w, Byte h, Byte r);
void SSD1306_FillRoundRect(Byte x, Byte y, Byte w, Byte h, Byte r);
void SSD1306_DrawTriangle(Byte x0, Byte y0, Byte x1, Byte y1, Byte x2, Byte y2);
void SSD1306_FillTriangle(Word x0, Word y0, Word x1, Word y1, Word x2, Word y2);
void SSD1306_PutC(Byte c);
void SSD1306_Print(char *s);
void SSD1306_PutCustomC(const Byte *c);
void SSD1306_DrawBMP(Byte x, Byte y, const Byte *bitmap, Byte w, Byte h);
void SSD1306_SetTextWrap(bool w);
void SSD1306_InvertDisplay(bool i);

//--------------------------------------------------------------------------//